

#
#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # 
  # Name :        K2_ToolsBiscuitJoint  ###
  # Date :	      July 2014
  # Version :     2.1.0
  # Author :      Keith Krueger (with a big help from plugin forum)
  #
 
require 'sketchup.rb'
require 'K2WS_Tools/k2ws_module.rb' #-NameSpace Module

module K²WS

  class K2_ToolsBiscuitJoint
      
    def initialize
      @mod = Sketchup.active_model
      @sel = @mod.selection
      @opt_LEN_No = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
      @opt_Format_No = Sketchup.active_model.options["UnitsOptions"]["LengthFormat"]
      @opt_Precision_No = Sketchup.active_model.options["UnitsOptions"]["LengthPrecision"]
      @ans=Array.new
      #Test for Acceptable Units/Format and Set rnd_Factor
      @rnd_Factor = 0
      if @opt_LEN_No == 0 && @opt_Format_No == 0 then @rnd_Factor = 10.0**-(1*@opt_Precision_No) end
      if @opt_LEN_No == 0 && @opt_Format_No == 3 then @rnd_Factor = 0.015625 * 2**(6 - @opt_Precision_No) end
      if @opt_LEN_No == 2 && @opt_Format_No == 0 then @rnd_Factor = 0.0393701 * 10**-(1*@opt_Precision_No) end
      if @rnd_Factor == 0
        UI.messagebox("Units must be Inches or Millimeters \n Fromat must be Decimal or Fractions \n \n CHANGE UNITS & RE-SELECT TOOL ")
        Sketchup.send_action "selectSelectionTool:"
      else
        #Debug        UI.messagebox("@rnd_Factor = " + @rnd_Factor.to_s)
      end #if @rnd_Factor
            
      @ip = Sketchup::InputPoint.new
      @no_sides = 12
            @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file)) 
        f = File.new(@file, "w+") 
        f.close
        self.reset
      else
        self.reset
        self.read_data
      end # FileTest
    end  #def initialize
	
    def read_data
      # Read an array of Input Data from the file 
      # open input data file in read mode	       
      f = File.open(@file, "r")    
      @ans = f.readlines
      #strip end of line or file from all data
      @ans = @ans.collect { |data|
        data.strip 
      }
      # Close the file
      f.close		
    end #read_data
    
    def reset
      @best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @addBiscuits = "No"
      @add_BiscuitSlots_Comp2 = "No"; @add_BiscuitSlots_Comp1 = "No"
    end
    
    def write_data
      # Write an array of Input Data to the file
      # open input data file in write mode	       
      f = File.open(@file, "w")
      f.puts @ans
      # Close the file
      f.close
    end 
    
    def readExistingData
      self.read_data
      
      @biscuitSize = @ans[0]
      @spacing = @ans[1]      
      @quanity = @ans[2].to_i
      @edge2CL = @ans[3].to_l
      @end2CL = @ans[4].to_l
      @end2Biscuit = @ans[5].to_l
      #ans 6 to 12 open
      @edge1_Last = @ans[13].to_l
      @edge2_Last = @ans[14].to_l
      #@ans[15] open
      #Get offset data
      self.no_of_Offset_Input
      n = @noOffsets
      @cl_offset=Array.new n
      i = 1
      i2 = 2
      while n > 0
        @cl_offset[i-1]=(@ans[15+i].to_s.strip.to_l)     
        n = n-1
        i = i+1
        i2 = i2+1		
      end #while n > 0
      
       
    end #readExistingData
    
    def same_Joint
      begin
        @edge1_Last = @ans[13].to_l; @edge2_Last = @ans[14].to_l
      rescue
        @edge1_Last = 0; @edge2_Last = 0
        #Debug UI.messagebox("blank data error rescued")
      end
      if @e_len1 == @edge1_Last && @e_len2 == @edge2_Last
        result = UI.messagebox('Use Existing Biscuit Joint Data ???', MB_YESNO)
        if result == 6 
          @use_ExistingData = "Yes" # @use_ExistingData is not Saved (Secesion only data)
        else
          @use_ExistingData = "No"  # @use_ExistingData is not Saved (Secesion only data)
        end # if result
      else
        @use_ExistingData = "No"  # @use_ExistingData is not Saved (Secesion only data)
      end # if @e_len1
      # Save current edge1 & edge2 Length
      @ans[13] = @e_len1.to_s;  @ans[14] = @e_len2.to_s
      write_data
    end # same_Joint 
    
    def select_Biscuit
      prompts = ["Joint Face READ ONLY  ","Select Biscuit","Quanity"]
      defaults = [@e_len2.to_s + " x " + @e_len1.to_s,@ans[0],@ans[2].to_s]
      choices = ["","#20|#10|#0|FF",""]
      results = inputbox prompts, defaults, choices, "Select Biscuit and Quanity"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @biscuitSize = results[1]; @ans[0] = results[1]
        self.biscuit_Length
        self.end_spacing
      @quanity = results[2].to_i; @ans[2] = results[2]
      write_data    
  
    end # select_Biscuit
  
    def end_spacing
      if @opt_LEN_No == 0 then @end2Biscuit = 0.250.inch end
      if @opt_LEN_No == 2 then @end2Biscuit = 6.mm end
    end #end_spacing
  
  
    def data_Input_Form
    
      # open input data file in read mode	       
      self.read_data
      cl_edge1 = @e_len1/2.0; cl_edge2 = round_to(@e_len2/2.0); end_toCL_first = round_to((@bl/2.0) + @end2Biscuit)
      if @quanity == 1
        prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Biscuit","End to CL Biscuit"]
        defaults= [@e_len2.to_s + " x " + @e_len1.to_s,cl_edge2.to_s,cl_edge1.to_s]
        choices = ["","",""]
      else
        prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Biscuits","End to CL First Biscuit","Space Between Biscuits"]
        defaults= [@e_len2.to_s + " x " + @e_len1.to_s,cl_edge2.to_s,end_toCL_first.to_s,@ans[1].to_s]
        choices = ["","","","Auto|User"]
      end # if @quanity
      results = inputbox prompts, defaults, choices, "Input Biscuit Joint Data (Biscuit: " + @biscuitSize + "  L = " + @bl.to_s + ")"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @edge2CL = results[1].to_l; @ans[3] = results[1]
      @end2CL = results[2].to_l;  @ans[4] = results[2]
      if @quanity == 1
        @spacing = "Auto"
      else
        @spacing = results[3]; @ans[1] = results[3]
      end
      # Write an array of Input Data to the file
      self.write_data
      
    end #data_Input_Form
    
    def select_Tool_Options
      @add_BiscuitSlots_Comp1 = "No"
      @add_BiscuitSlots_Comp2 = "No"
      @addBiscuits = "No"
      prompts = ["Select Tool Option"]
      defaults= ["Complete Joint"]
      choices = ["Complete Joint|Comp 1 Slots + Biscuits|Add Biscuits|Comp1 Slots|Comp 2 Slots|Comp 1 & 2 Slots|comp 2 Slots + Biscuits_"]
      results = inputbox prompts, defaults, choices, "Select Biscuit Joint Tool Options"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      case results[0]
        when "Complete Joint"
          @add_BiscuitSlots_Comp1 = "Yes"
          @add_BiscuitSlots_Comp2 = "Yes"
          @addBiscuits = "Yes"
        when "Comp 1 Slots + Biscuits"
          @add_BiscuitSlots_Comp1 = "Yes"
          @addBiscuits = "Yes"
        when "Add Biscuits"
          @addBiscuits = "Yes"
        when "Comp1 Slots"
          @add_BiscuitSlots_Comp1 = "Yes"
        when "Comp 2 Slots"
          @add_BiscuitSlots_Comp2 = "Yes"
        when "Comp 1 & 2 Slots"
          @add_BiscuitSlots_Comp1 = "Yes"
          @add_BiscuitSlots_Comp2 = "Yes"
        when "comp 2 Slots + Biscuits_"
          @add_BiscuitSlots_Comp2 = "Yes"
          @addBiscuits = "Yes"
      end # case results
    end # select_Tool_Options
  
    def onCancel(flag, view)
      view.refresh if @drawn
      Sketchup.send_action "selectSelectionTool:"
    end

    def draw(view)
      @drawn = false
      if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
      end
    end

    def no_of_Offset_Input
      #Debug UI.messagebox("spacing  =  " + @spacing.to_s)
      case @spacing    
        when "Auto"
          @noOffsets=0
        when "User"
          @noOffsets=@quanity - 1
      end #case @spacing
      self.write_data 
    end #no_of_offsets
        
    def round_to(num)
      #Debug      UI.messagebox("origional no =  " + num.to_s)
      num_R = ((num + (@rnd_Factor/2.0))/@rnd_Factor).to_i*@rnd_Factor
      num_R = num_R.to_l
      #Debug      UI.messagebox("rounded no =  " + num_R.to_s)
      return num_R
    end #round_to  
    
      
    def round_3places(n)  
      n=(n*1000).to_i
      n=n.to_f/1000      
    end #def round_3places
    
    def input_Offsets
      # open a file in read mode	       
      self.read_data
      n = @noOffsets
      @cl_offset=Array.new n
      i = 1
      i2 = 2
      space = round_to(@e_len1 - @end2CL)
      while n > 0
        prompts = ["CL Distance Biscuit " + i.to_s + " to Biscuit " + i2.to_s + " ","Remaining Space   READ ONLY  ","Number Spaces   READ ONLY  "]
        defaults= [@ans[15+i].to_s,space.to_s,n.to_s]
        results = inputbox prompts, defaults, "Input CL Distance between Biscuits"
        @cl_offset[i-1]=results[0].to_l; @ans[15+i] = results[0]
        space = round_to(space - @cl_offset[i-1])
        n = n-1
        i = i+1
        i2 = i2+1		
      end #while n > 0
      # Write an array of Input Data to the file
      self.write_data      
    end #def input_Offsets 

    def cal_xCL_points(x1,e_len,dir_x)
      @xcl=Array.new @quanity.to_i
      case @spacing           
        when "Auto"
          #Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
          equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
          @xcl[0]=x1+(@end2CL*dir_x)
          @xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
          n =(@quanity-2)  #loop number
          sn = 1 #Starting number
          while n > 0
            @xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
            sn = sn + 1
            n = n -1
          end # while n > 0
        #Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
        when "User"
          #Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
          @xcl[0]=x1+(@end2CL*dir_x)
          n =(@quanity-1)  #loop number
          sn = 1 #Starting number
          i = 0
          while n > 0
            user_offset=@cl_offset[i]
            #user_offset=user_offset.to_l
            #Debug  UI.messagebox("@xcl[sn-1] = "+ @xcl[sn-1].to_s)
            #Debug  UI.messagebox("user_offset = "+ user_offset.to_s)
            @xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
            sn = sn + 1
            i = i + 1
            n = n -1
          end # while n > 0
        #Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)}         
      end #case @spacing
    end #cal_xCL_points
    
    def biscuit_Length
      case @biscuitSize
        when "#20"
          @bl=2.626
        when "#10"
          @bl=2.438
        when "#0"
          @bl=2.188
        when "FF"
          @bl=1.312
      end #case @biscuitSize
    
    end #biscuit_Length
    
    def points_BiscuitSlot(x,y,z,dz_dir)
      @pt=Array.new 16
      w=(0.156/2).inch
      case @biscuitSize
        when "#20"
          dx1=1.313.inch;	dx2=0.922.inch;	dz2=0.276.inch;	dx3=0.475.inch;	dz3=0.449.inch;	dz4=0.508.inch
        when "#10"
          dx1=1.219.inch;	dx2=0.850.inch;	dz2=0.238.inch;	dx3=0.436.inch;	dz3=0.386.inch;	dz4=0.436.inch
        when "#0"
          dx1=1.094.inch;	dx2=0.754.inch;	dz2=0.184.inch;	dx3=0.384.inch;	dz3=0.297.inch;	dz4=0.335.inch
        when "FF"
          dx1=0.656.inch;	dx2=0.461.inch;	dz2=0.138.inch;	dx3=0.238.inch;	dz3=0.224.inch;	dz4=0.254.inch
      end #case @biscuitSize
      dz2 = dz2 * dz_dir
      dz3 = dz3 * dz_dir
      dz4 = dz4 * dz_dir
      #Top Arc
      #Debug      UI.messagebox("x = " + x.to_s);UI.messagebox("dx1 = " + dx1.to_s);UI.messagebox("y = " + y.to_s)
      @pt[0]=Geom::Point3d.new(x-dx1,y+w,z)
      @pt[1]=Geom::Point3d.new(x-dx2,y+w,z-dz2)
      @pt[2]=Geom::Point3d.new(x-dx3,y+w,z-dz3)
      @pt[3]=Geom::Point3d.new(x,y+w,z-dz4)
      @pt[4]=Geom::Point3d.new(x+dx3,y+w,z-dz3)
      @pt[5]=Geom::Point3d.new(x+dx2,y+w,z-dz2)
      @pt[6]=Geom::Point3d.new(x+dx1,y+w,z)
      #Bottom Arc
      @pt[7]=Geom::Point3d.new(x-dx1,y-w,z)
      @pt[8]=Geom::Point3d.new(x-dx2,y-w,z-dz2)
      @pt[9]=Geom::Point3d.new(x-dx3,y-w,z-dz3)
      @pt[10]=Geom::Point3d.new(x,y-w,z-dz4)
      @pt[11]=Geom::Point3d.new(x+dx3,y-w,z-dz3)
      @pt[12]=Geom::Point3d.new(x+dx2,y-w,z-dz2)
      @pt[13]=Geom::Point3d.new(x+dx1,y-w,z)
      #Center point for Tringular Faces top 
      @pt[14]=Geom::Point3d.new(x,y+w,z)
      #Center point for Tringular Faces bottom
      @pt[15]=Geom::Point3d.new(x,y-w,z)   
      #Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
    end #points_BiscuitSlot
    
    def biscuit_name
      #Biscuit name based on Size
      @biscuit_name = ("Part Biscuit_" + @biscuitSize)
      #Debug UI.messagebox("Biscuit Name = " + @biscuit_name)
    end #biscuit_name

    def find_BiscuitComp(model,entities,definitions)
      length = definitions.length
      dname = ""
      i=0
      loop do
        i += 1
        break if i==length
        definition = definitions[length-i]
        dname = definition.name
        break if dname == @biscuit_name
      end  
        if dname == @biscuit_name
        @biscuit_exist = "yes"
        @biscuit_definition = definitions[length-i]
      end #if dname
    end #find_BiscuitComp 
    
    def points_Biscuit(x,y,z)
      @ptbb=Array.new 14
      @ptbt=Array.new 14
      w=(0.156/2).inch
      case @biscuitSize
        when "#20"
          dx1b=1.110.inch; dz1b=0.113; dx2b=0.756.inch; dz2b=0.302.inch; dx3b=0.383.inch;	dz3b=0.417.inch; dz4b=0.456.inch
        when "#10"
          dx1b=1.051.inch; dz1b=0.075; dx2b=0.716.inch; dz2b=0.231.inch; dx3b=0.363.inch;	dz3b=0.326.inch; dz4b=0.358.inch
        when "#0"
          dx1b=0.932.inch; dz1b=0.074; dx2b=0.635.inch; dz2b=0.197.inch; dx3b=0.322.inch;	dz3b=0.272.inch; dz4b=0.297.inch
        when "FF"
          dx1b=0.562.inch; dz1b=0.030; dx2b=0.446.inch; dz2b=0.111.inch; dx3b=0.230.inch;	dz3b=0.195.inch; dz4b=0.224.inch
      end #case @biscuitSize
      
      #Top Face     
      @ptbt[0]=Geom::Point3d.new(x-dx1b,y+w,z-dz1b)
      @ptbt[1]=Geom::Point3d.new(x-dx2b,y+w,z-dz2b)
      @ptbt[2]=Geom::Point3d.new(x-dx3b,y+w,z-dz3b)
      @ptbt[3]=Geom::Point3d.new(x,y+w,z-dz4b)
      @ptbt[4]=Geom::Point3d.new(x+dx3b,y+w,z-dz3b)
      @ptbt[5]=Geom::Point3d.new(x+dx2b,y+w,z-dz2b)
      @ptbt[6]=Geom::Point3d.new(x+dx1b,y+w,z-dz1b)
      @ptbt[7]=Geom::Point3d.new(x+dx1b,y+w,z+dz1b)
      @ptbt[8]=Geom::Point3d.new(x+dx2b,y+w,z+dz2b)
      @ptbt[9]=Geom::Point3d.new(x+dx3b,y+w,z+dz3b)
      @ptbt[10]=Geom::Point3d.new(x,y+w,z+dz4b)
      @ptbt[11]=Geom::Point3d.new(x-dx3b,y+w,z+dz3b)
      @ptbt[12]=Geom::Point3d.new(x-dx2b,y+w,z+dz2b)
      @ptbt[13]=Geom::Point3d.new(x-dx1b,y+w,z+dz1b)
      #Bottom Face
      @ptbb[0]=Geom::Point3d.new(x-dx1b,y-w,z-dz1b)
      @ptbb[1]=Geom::Point3d.new(x-dx2b,y-w,z-dz2b)
      @ptbb[2]=Geom::Point3d.new(x-dx3b,y-w,z-dz3b)
      @ptbb[3]=Geom::Point3d.new(x,y-w,z-dz4b)
      @ptbb[4]=Geom::Point3d.new(x+dx3b,y-w,z-dz3b)
      @ptbb[5]=Geom::Point3d.new(x+dx2b,y-w,z-dz2b)
      @ptbb[6]=Geom::Point3d.new(x+dx1b,y-w,z-dz1b)
      @ptbb[7]=Geom::Point3d.new(x+dx1b,y-w,z+dz1b)
      @ptbb[8]=Geom::Point3d.new(x+dx2b,y-w,z+dz2b)
      @ptbb[9]=Geom::Point3d.new(x+dx3b,y-w,z+dz3b)
      @ptbb[10]=Geom::Point3d.new(x,y-w,z+dz4b)
      @ptbb[11]=Geom::Point3d.new(x-dx3b,y-w,z+dz3b)
      @ptbb[12]=Geom::Point3d.new(x-dx2b,y-w,z+dz2b)
      @ptbb[13]=Geom::Point3d.new(x-dx1b,y-w,z+dz1b)   
      #Debug @ptbt.each {|e| UI.messagebox("point biscuit top = "+ e.to_s)}
      #Debug @ptbb.each {|e| UI.messagebox("point biscuit bot = "+ e.to_s)}    
    end #points_Biscuit
    
    def draw_Biscuit(entities)
      #Draws Biscuit Comp @0,0,0
      #Debug UI.messagebox("Starting Biscuit")
      points_Biscuit(0,0,0)
      #Add Biscuit
      bface=entities.add_face(@ptbt)
      bface.reverse!
      bface=entities.add_face(@ptbb)
      
      for k in -1...13
        k2=k+1
        bface=entities.add_face(@ptbb[k],@ptbt[k],@ptbt[k2],@ptbb[k2])
      end #for k
      #Debug      UI.messagebox("Created Biscuit")
    
    end #draw_Biscuit
    
    def points_BiscuitCL(x,y)
      @clpt=Array.new 4
      cline=0.03.inch
      @clpt[0]=Geom::Point3d.new(x,y,0)
      @clpt[1]=Geom::Point3d.new(x+(cline),y,0)
      @clpt[2]=Geom::Point3d.new(x,y+(cline),0)
      @clpt[3]=Geom::Point3d.new(x,y,cline)
      #Debug @clpt.each {|e| UI.messagebox("CL point Biscuit = "+ e.to_s)} 
    end #def points_BiscuitCL
    
    def onMouseMove(flags, x, y, view)
      Sketchup::set_status_text("Select 1st Component for Biscuit Joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
      Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1 && @toolType == "Joint_Tool"
    Sketchup::set_status_text("Select 2nd Component to Hide:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1 && @toolType == "SingleComp_Tool"
      Sketchup::set_status_text("Select First Edge in component 1 Parallel to Biscuit Length:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
      Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
      @ip.pick view,x,y
      view.tooltip = @ip.tooltip     
      view.refresh
    end
    
    def onLButtonDown(flags, x, y, view)
      if @status==0 #select 1st component
        ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
        if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
          @sel.add(@comp1); @ent1=@comp1.definition.entities; @c1_layer = @comp1.layer
          #Debug UI.messagebox("Component 1 Layer = " + @c1_layer.to_s)
          @status=1
          onMouseMove(flags, x, y, view)
        else
          UI.messagebox "Selection invalid, must be component"
        end # if@comp1			
      elsif @status==1	#select 2nd component
        ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
        if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
          @sel.add(@comp2); @ent2=@comp2.definition.entities;  @status=2
          @tform2 = @comp2.transformation.inverse
          onMouseMove(flags, x, y, view)
          @sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
        else
          UI.messagebox "Selection invalid, must be component"
        end #@entity			
      elsif @status==2 #select long edge, width
        @ip.pick view,x,y
        if(@ip.valid?)
          if(@ip.edge && @ent1.include?(@ip.edge))
            @tform = @ip.transformation
            @edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position; @e_len1=@edge1.length 
            @status = 3
            onMouseMove(flags, x, y, view)
          else
            UI.messagebox "Selection invalid Use Right Click & Hide Interfering Comp"
          end #if(@ip.edge
        else
          UI.messagebox "Selection invalid "	
        end #if(@ip.valid?)
      elsif @status==3 #select short edge, thickness
        @ip.pick view,x,y
        if(@ip.valid?)
          if(@ip.edge && @ent1.include?(@ip.edge))
            @edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position; @e_len2=@edge2.length   
            if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
              orig=@e1sp
              p1=@e1sp
              p2=@e1ep
            end #if ((@e1sp==
            if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
              orig=@e1ep
              p2=@e1sp
              p1=@e1ep
            end  #if ((@e1ep==
            if orig==@e2sp
              p3=@e2ep
              else
              p3=@e2sp
            end #if orig

            #Data Input Section
            self.same_Joint
            #Debug UI.messagebox("exited same joint")
            if @use_ExistingData == "No"
              self.select_Biscuit
              self.data_Input_Form
              self.no_of_Offset_Input
              #Debug UI.messagebox("@noOffsets = " + @noOffsets.to_s)
              if @noOffsets >= 1 then input_Offsets end #@noOffsets
            else
              begin
                self.readExistingData
              rescue
                UI.messagebox("Problem reading data! ReEnter Data")
                self.select_Biscuit
                self.data_Input_Form
                self.no_of_Offset_Input
              end
            end # if @use_ExistingData
          
            #find face that has edges 1 and 2
            face=@edge1.common_face @edge2
            if (face==nil)#14                    #get valid face
              @oops=4
            else 
              #Debug UI.messagebox"got valid face"
              #get points in local coord system for face
              #xaxis parallel to @edge1
              x,y,z=face.normal.axes
              #get vector parallel to @edge1              			         			          
              ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
              xx=ar.normalize
              yy=z.cross xx
              localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
              backt=Geom::Transformation.axes(orig,xx,yy,z)
              edges=face.outer_loop.edges
              eind=edges.index(@edge1)
              e_len=@edge1.length
              #Points on face in local coordinates
              p1=p1.transform(localt)
              p2=p2.transform(localt)
              p3=p3.transform(localt)           
              x1=p1.x
              x2=p2.x
              y1=p1.y
              y2=p3.y
              if x1>x2 then dir_x = -1 end
              if x1<x2 then dir_x = +1 end
              if y1<y2 then dir_y = +1 end
              if y1>y2 then dir_y = -1 end						                                                                       
              cal_xCL_points(x1,e_len,dir_x) # Returns @xcl=Array.new @quanity 
              #Debug @xcl.each {|element| UI.messagebox("Cal @xcl Points = "+ element.to_s)}
              @ycl = y1+(@edge2CL*dir_y)
              
              # Options for Complete Joint or Each Part of the Joint
              select_Tool_Options
              if @add_BiscuitSlots_Comp1 == "Yes" 
                Sketchup.active_model.start_operation "Biscuit Slots Comp 1"
                i=0
                n=@quanity
                while n > 0               			          
                  #Debug UI.messagebox("yc1 = "+ @ycl.to_s)
                  #Calculate points for Biscuit Slot
                  dz_dir = 1 # Component 1
                  points_BiscuitSlot(@xcl[i],@ycl,0,dz_dir) # Returns @pt=Array.new 16
                  #Debug @pt.each {|element| UI.messagebox("@pt point comp 1 = "+ element.to_s)}               
                  #Transform Biscuit Slot Points to Component 1 Coordinates
                  @pt=@pt.collect {|point| point.transform(backt)} #Transform to Comp 1               
                  #Debug @pt.each {|element| UI.messagebox("@pt point comp 1 = "+ element.to_s)}
                  #Add Biscuit Slot
                  #if @toolType=="Joint_Tool" then @comp2.hidden=false  end
                  #@comp1.hidden=false
                  @entity=@comp1
                  codef=@entity.definition
                  @sel.clear
                  @sel.add(@comp1)
                  # CREATING SLOT IN COMPONENT 1
                  #Add Slot Face then Delete to create slot opening
                  cface=codef.entities.add_face(@pt[0],@pt[6],@pt[13],@pt[7])
                  codef.entities.erase_entities cface                
                  #Loop for adding faces needed to create slot
                  for k in 0..5
                    k1=k+1
                    k8=k+8
                    k7=k+7                                                
                    #Add back curved face of slot
                    cface=codef.entities.add_face(@pt[k],@pt[k1],@pt[k8],@pt[k7])                                
                    #Add  face to top of slot
                    cface=codef.entities.add_face(@pt[0..6])
                    #Add  face to bottom of slot
                    cface=codef.entities.add_face(@pt[7..13])
                  end #for k in 0..5
                  i=i+1
                  n=n-1
                end # while n
                # Biscuit Slots added to First component at this point
                Sketchup.active_model.commit_operation
              end # add_BiscuitSlots_Comp1             
                
                if @add_BiscuitSlots_Comp2 == "Yes" 
                Sketchup.active_model.start_operation "Biscuit Slots Comp 2"
                i=0
                n=@quanity
                while n > 0   
                  @entity=@comp2
                  codef=@entity.definition
                  @sel.clear
                  @sel.add(@comp2)
                  @statusstring= "Adding Feature to Second Component"
                  Sketchup::set_status_text @statusstring,SB_PROMPT
                  #Calculate points for Biscuit Slot in comp 2
                  dz_dir = -1 # Component 2
                  points_BiscuitSlot(@xcl[i],@ycl,0,dz_dir) # Returns @pt=Array.new 16
                  @pt=@pt.collect {|point| point.transform(backt)} # Transform to Comp1
                  @ptg=@pt.collect {|point| point.transform(@tform)} #Transform to Global
                  @ptC2=@ptg.collect {|point| point.transform(@tform2)} #Transform to Comp 2
                  # CREATING SLOT IN COMPONENT 2
                  #Add Slot Face then Delete to create slot opening
                  cface=codef.entities.add_face(@ptC2[0],@ptC2[6],@ptC2[13],@ptC2[7])
                  codef.entities.erase_entities cface                
                  #Loop for adding faces needed to create slot
                  for k in 0..5
                    k1=k+1
                    k8=k+8
                    k7=k+7                                                
                    #Add back curved face of slot
                    cface=codef.entities.add_face(@ptC2[k],@ptC2[k1],@ptC2[k8],@ptC2[k7])                                
                    #Add  face to top of slot
                    cface=codef.entities.add_face(@ptC2[0..6])
                    #Add  face to bottom of slot
                    cface=codef.entities.add_face(@ptC2[7..13])
                  end #for i in 0..5                
                  i=i+1
                  n=n-1
                end #while n > 0
                # Biscuit Slots added to Second component at this point
                Sketchup.active_model.commit_operation
              end # add_BiscuitSlots_Comp2
                            
              if @addBiscuits == "Yes" 
                @sel.clear
                Sketchup.active_model.start_operation "Biscuits"
                model = Sketchup.active_model
                entities = model.active_entities
                definitions = model.definitions
                layers = model.layers
                group = entities.add_group
                entities = group.entities
                # Get existing biscuit or Draw a biscuit Component
                biscuit_name
                @biscuit_exist = "no"
                find_BiscuitComp(model,entities,definitions)
                if @biscuit_exist == "no" #biscuit component does not exist
                  draw_Biscuit(entities) #Adds biscuit @(0,0,0)
                  component_instance=group.to_component
                  component_instance.definition.name=(@biscuit_name)
                  definition = component_instance.definition
                else
                  definition = @biscuit_definition
                end #biscuit_exist
                i=0
                n=@quanity
                while n > 0
                  @ycl = y1+(@edge2CL*dir_y)
                  #Calculate z for correct biscuit position
                  #Calculate CL of Slot
                  #Debug UI.messagebox("ycl " + @ycl.to_s)
                  #Debug UI.messagebox("@xcl[i] " + @xcl[i].to_s)
                  biscuit_ip=Geom::Point3d.new([@xcl[i],@ycl,0])								
                  biscuit_ip=biscuit_ip.transform(backt)					
                  biscuit_ip=biscuit_ip.transform(@tform)
                  #Get points for biscuit axis at center of hole
                  points_BiscuitCL(@xcl[i],@ycl)
                  @clpt=@clpt.collect {|point| point.transform(backt)}
                  @clpt=@clpt.collect {|point| point.transform(@tform)}
                  #Debug @clpt.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}
                  #Create vectors for biscuit axis
                  x_vector = @clpt[0].vector_to @clpt[1]
                  y_vector = @clpt[0].vector_to @clpt[2]
                  z_vector = @clpt[0].vector_to @clpt[3]
                  #Invoke Geom::Transformation.axes with the new origin, x-axis, y-axis, and z-axis
                  f_tran = Geom::Transformation.axes(biscuit_ip,x_vector,y_vector,z_vector)
                  #For i = 0 move existing component to first hole!
                  if i == 0 && @biscuit_exist == "no"
                    #Debug Error to Stop Program  UI.messagebox("f_tran = " + f_tran)
                    component_instance.transform! f_tran
                    component_instance.layer = @c1_layer
                  else
                    if i == 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                    if i == 0 then instance2.layer = @c1_layer end
                  end
                  #For i > 0 Insert new instance of biscuit at biscuit axis origin
                  if i > 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                  if i > 0 then instance2.layer = @c1_layer end
                  i=i+1
                  n=n-1
                end #while
                #Biscuits added to Joint at this point
                Sketchup.active_model.commit_operation
              end # addBiscuits
            @comp2.hidden=false
            #Program is Complete and ends
            Sketchup.send_action "selectSelectionTool:"          
            end #face==nil                        
          end #if(@ip.edge
        else
          UI.messagebox "Selection invalid"	
        end #if(@ip.valid?)	
      end #elsif @status==
    end # def onLButtonDown

    def onCancel(flag, view)
      view.refresh if @drawn
      Sketchup.send_action "selectSelectionTool:"
    end

    def draw(view)
      @drawn = false
      if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
      end
    end
    
    def onRButtonDown(flags,x,y,view)
      reset
      self.data_Input_Form
      self.no_of_Offset_Input
      if @noOffsets >= 1
        input_Offsets
      end
    end
    
  end # class K2_ToolsBiscuitJoint

  
  
 end # module
 
file_loaded("K2_ToolsBiscuitJoint.rb") 