


#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # 
  # Name :        K2_ToolsDominoJoint.rb ###
  # Date :	      July 2014
  # Version :     2.1.0
  #
  #
  # Author :       Keith Krueger  (with a big help from plugin forum)
  #
 
require 'sketchup.rb'
require 'K2WS_Tools/k2ws_module.rb' #-NameSpace Module

module K²WS
 
class K2_Tools_DominoJoint 
    
	def initialize
      @mod = Sketchup.active_model
      @sel = @mod.selection
      @opt_LEN_No = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
      @opt_Format_No = Sketchup.active_model.options["UnitsOptions"]["LengthFormat"]
      @opt_Precision_No = Sketchup.active_model.options["UnitsOptions"]["LengthPrecision"]
      @ans=Array.new
      #Test for Acceptable Units/Format and Set rnd_Factor
      @rnd_Factor = 0
      if @opt_LEN_No == 0 && @opt_Format_No == 0 then @rnd_Factor = 10.0**-(1*@opt_Precision_No) end
      if @opt_LEN_No == 0 && @opt_Format_No == 3 then @rnd_Factor = 0.015625 * 2**(6 - @opt_Precision_No) end
      if @opt_LEN_No == 2 && @opt_Format_No == 0 then @rnd_Factor = 0.0393701 * 10**-(1*@opt_Precision_No) end
      if @rnd_Factor == 0
        UI.messagebox("Units must be Inches or Millimeters \n Fromat must be Decimal or Fractions \n \n CHANGE UNITS & RE-SELECT TOOL ")
        Sketchup.send_action "selectSelectionTool:"
      else
        #Debug        UI.messagebox("@rnd_Factor = " + @rnd_Factor.to_s)
      end #if @rnd_Factor
            
      @ip = Sketchup::InputPoint.new
      @dowel_Version = "2.2.6"
      @no_sides = 12
      @base_path = Sketchup.find_support_file("Plugins")
      @rel_path = "K2WS_Tools/K2_ToolsSupportFiles"
      help_path = File.join(@base_path,@rel_path)
      @helpfile = "K2_ToolsDowelHelp.jpg"
      @fullpath = File.join(@base_path,@rel_path,@helpfile)
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file)) 
        f = File.new(@file, "w+") 
        f.close
        self.reset
      else
        self.reset
        self.read_data
      end # FileTest
    end  #def initialize
	
	def read_data
      # Read an array of Input Data from the file 
      # open input data file in read mode	       
      f = File.open(@file, "r")    
      @ans = f.readlines
      #strip end of line or file from all data
      @ans = @ans.collect { |data|
        data.strip 
      }
      # Close the file
      f.close		
    end #read_data
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		
		@dominoSize = @ans[0] 		
		@spacing_Type = @ans[1]
		@widthSetting = @ans[2]                 
		@quanity = @ans[3].to_i                 
		@edge2CL = @ans[4].to_l                 
		@end2CL = @ans[5].to_l 		
		@mdC1 = @ans[6].to_l      
		@mdC2 = @ans[7].to_l
    @d_Comp1 = @ans[8].to_l
    @d_length = @ans[9].to_l
    @domino_name = @ans[10]
    @d_Comp2 = @ans[11].to_l
    @thru_Mortise = @ans[12]
    @edge1_Last = @ans[13].to_l
    @edge2_Last = @ans[14].to_l
    @use_ExistingData = @ans[15]
    
		 self.no_of_Offset_Input
      n = @noOffsets
      @cl_offset=Array.new n
      i = 1
      i2 = 2
      while n > 0
        @cl_offset[i-1]=(@ans[15+i].to_s.strip.to_l)     
        n = n-1
        i = i+1
        i2 = i2+1		
      end #while n > 0
	end #readExistingData
	
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @addDomino = "No"
    @add_DomSlots_Comp2 = "No"; @add_DomSlots_Comp1 = "No"
	end
  
  def same_Joint
      begin
        @edge1_Last = @ans[13].to_l; @edge2_Last = @ans[14].to_l
      rescue
        @edge1_Last = 0; @edge2_Last = 0
        #Debug UI.messagebox("blank data error rescued")
      end
      if @e_len1 == @edge1_Last && @e_len2 == @edge2_Last
        result = UI.messagebox('Use Existing Domino Joint Data ???', MB_YESNO)
        if result == 6 
          @use_ExistingData = "Yes" # @use_ExistingData is not Saved (Secesion only data)
        else
          @use_ExistingData = "No"  # @use_ExistingData is not Saved (Secesion only data)
        end # if result
      else
        @use_ExistingData = "No"  # @use_ExistingData is not Saved (Secesion only data)
      end # if @e_len1
      # Save current edge1 & edge2 Length
      @ans[13] = @e_len1.to_s;  @ans[14] = @e_len2.to_s
      write_data
    end # same_Joint
  
  def select_Domino
    prompts = ["Joint Face READ ONLY  ","Select Domino", "Mortise Width Setting","Quanity"]
    defaults = [@e_len2.to_s + " x " + @e_len1.to_s,@ans[0],@ans[2].to_s,@ans[3].to_s]
    choices = ["","4 x 20|5 x 30|6 x 40|8 x 40|8 x 50|8 x 80|8 x 100|10 x50|10 x 80|10 x 100|12 x 100|12 x 140|14 x 100|14 x 140","+0mm|+3mm|+6mm|+10mm",""]
    results = inputbox prompts, defaults, choices, "Select Domino and Mortise Width"
    if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    @dominoSize = results[1]; @ans[0] = results[1]
    @widthSetting = results[2] ; @ans[2] = results[2]
    @quanity = results[3].to_i; @ans[3] = results[3]
    #Get diminsion data for selected domino
    dominoDimensions
    @d_length = @lt; @ans[9] = @lt.to_s
    #Debug  UI.messagebox("Domino Length =  " + @d_length.to_s)
    write_data    
  
  end # select_Domino
  
	def data_Input_Form
  
		# open input data file in read mode	       
		self.read_data
    cl_edge1 = @e_len1/2; cl_edge2 = round_to(@e_len2/2.0); end_toCL_first = round_to(((@wm/2.0)+@tt))
    if @quanity == 1
      prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Domino","End to CL Domino","Domino Depth Comp1","Through Mortise? "]
      defaults= [@e_len2.to_s + " x " + @e_le1.to_s,cl_edge2.to_s,cl_edge1.to_s,@ans[8].to_s,@ans[12]]
      choices = ["","","","","No|Comp 1|Comp 2|Both Comp"]
    else
      prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Dominos","End to CL First Domino","Domino Depth Comp1","Through Mortise? ","Space Between Dominos"]
      defaults= [@e_len2.to_s + " x " + @e_len1.to_s,cl_edge2.to_s,end_toCL_first.to_s,@ans[8].to_s,@ans[12].to_s,@ans[1]]
      choices = ["","","","","No|Comp 1|Comp 2|Both Comp","Auto|User"]
    end # if @quanity
		results = inputbox prompts, defaults, choices, "Input Data for Domino Joint (Domino: " + @dominoSize +")"
		if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    @edge2CL = results[1].to_l; @ans[4] = results[1]
    @end2CL = results[2].to_l; @ans[5] = results[2]
    @d_Comp1 = results[3].to_l; @ans[8] = results[3]; @d_Comp2 = (@d_length - @d_Comp1); @ans[11] = @d_Comp2.to_s
    @thru_Mortise = results[4]; @ans[12] = results[4]
    if @quanity == 1
      @spacing_Type = "Auto"
      #Debug UI.messagebox("spacing_Type set to  " + @spacing)
    else
      @spacing_Type = results[5]; @ans[1] = results[5]                          
    end #if @quanity
    if @thru_Mortise == "No"
      @mdC1 = @d_Comp1; @ans[6] = @mdC1.to_s
      @mdC2 = @d_length - @d_Comp1; @ans[7] = @mdC2.to_s
    else
      mortise_Thru_Depth
    end # if @thru_Mortise  
		# Write an array of Input Data to the file
		self.write_data
    
	end #data_Input_Form
   

	def mortise_Thru_Depth

		if @thru_Mortise == "Comp 1"
      prompts = ["Depth Thru Mortise Comp 1 "]
      defaults= [@ans[6].to_s]
      choices = [""]
      results = inputbox prompts, defaults, choices, "Through Mortise Depth COMPONENT 1"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @mdC1 = results[0].to_l; @ans[6] = results[0]
      @mdC2 = @d_length - @d_Comp1; @ans[7] = @mdC2.to_s
    elsif @thru_Mortise == "Comp 2"
      prompts = ["Depth Thru Mortise Comp 2 "]
      defaults= [@ans[7].to_s]
      choices = [""]
      results = inputbox prompts, defaults, choices, "Through Mortise Depth COMPONENT 2"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @mdC1 = @d_Comp1; @ans[6] = @mdC1.to_s
      @mdC2 = results[0].to_l; @ans[7] = results[0]
    elsif @thru_Mortise == "Both Comp"
      prompts = ["Depth Thru Mortise Comp 1 ","Depth Thru Mortise Comp 2 "]
      defaults= [@ans[6].to_s,@ans[7].to_s]
      choices = ["",""]
      results = inputbox prompts, defaults, choices, "Through Mortise Depth Both COMPONENTS"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @mdC1 = results[0].to_l; @ans[6] = results[0]
      @mdC2 = results[1].to_l; @ans[7] = results[1]
    end #@thru_Mortise
		# open input data file in write mode	       
		self.write_data
    
	end #mortise_Thru_Depth
  
  def select_Tool_Options
    @add_DomSlots_Comp1 = "No"
    @add_DomSlots_Comp2 = "No"
    @addDomino = "No"
    prompts = ["Select Tool Option"]
    defaults= ["Complete Joint"]
    choices = ["Complete Joint|Comp 1 Slots + Dominos|Add Dominos|Comp1 Slots|Comp 2 Slots|Comp 1 & 2 Slots|Comp 2 Slots + Dominos_"]
    results = inputbox prompts, defaults, choices, "Select Domino Joint Tool Options"
    if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    case results[0]
      when "Complete Joint"
        @add_DomSlots_Comp1 = "Yes"
        @add_DomSlots_Comp2 = "Yes"
        @addDomino = "Yes"
      when "Comp 1 Slots + Dominos"
        @add_DomSlots_Comp1 = "Yes"
        @addDomino = "Yes"
      when "Add Dominos"
        @addDomino = "Yes"
      when "Comp1 Slots"
        @add_DomSlots_Comp1 = "Yes"
      when "Comp 2 Slots"
        @add_DomSlots_Comp2 = "Yes"
      when "Comp 1 & 2 Slots"
        @add_DomSlots_Comp1 = "Yes"
        @add_DomSlots_Comp2 = "Yes"
      when "Comp 2 Slots + Dominos_"
        @add_DomSlots_Comp2 = "Yes"
        @addDomino = "Yes"
    end # case results
  end # select_Tool_Options
  
	def dominoDimensions
		case @dominoSize
    when "4 x 20"
			@tt=4.mm; @wt=17.5.mm; @lt=20.mm; 
		when "5 x 30"
			@tt=5.mm; @wt=18.5.mm; @lt=30.mm; 
		when "6 x 40"
			@tt=6.mm; @wt=19.5.mm; @lt=40.mm; 
		when "8 x 40"
			@tt=8.mm; @wt=21.5.mm; @lt=40.mm; 
		when "8 x 50"
			@tt=8.mm; @wt=21.5.mm; @lt=50.mm; 
		when "8 x 80"
			@tt=8.mm; @wt=21.5.mm; @lt=80.mm; 
    when "8 x 100"
			@tt=8.mm; @wt=21.5.mm; @lt=100.mm;
    when "10 x50"
			@tt=10.mm; @wt=23.5.mm; @lt=50.mm;
    when "10 x 80"
			@tt=10.mm; @wt=23.5.mm; @lt=80.mm;
    when "10 x 100"
			@tt=10.mm; @wt=23.5.mm; @lt=100.mm;
    when "12 x 100"
			@tt=12.mm; @wt=25.5.mm; @lt=100.mm;
    when "12 x 140"
			@tt=12.mm; @wt=25.5.mm; @lt=140.mm;
    when "14 x 100"
			@tt=14.mm; @wt=27.5.mm; @lt=100.mm;
    when "14 x 140"
			@tt=14.mm; @wt=27.5.mm; @lt=140.mm;
		end # case @dominoSize
    
    @domino_name = "Part Domino_" + @dominoSize; @ans[10] = @domino_name
    #Debug  UI.messagebox("domino_name =  " + domino_name) 
      
    case @widthSetting #Mortise Dim affected by width setting
		when "+0mm"
			@wm=@wt; @wf=@wm-@tt; @xf=@wf/2;
    when "+3mm"
			@wm=@wt+3.mm; @wf=@wm-@tt; @xf=@wf/2;       
		when "+6mm"
			@wm=@wt+6.mm; @wf=@wm-@tt; @xf=@wf/2
		when "+10mm"
			@wm=@wt+10.mm; @wf=@wm-@tt; @xf=@wf/2        
		end # case @widthSetting
		
  end #dominoDimensions
  
	def no_of_Offset_Input
      case @spacing_Type    
        when "Auto"
          @noOffsets = 0
        when "User"
          @noOffsets = @quanity - 1
      end #case @spacing_Type
      self.write_data
      #Debug UI.messagebox("No of Offsets =" + @noOffsets.to_s)
    end #no_of_offsets
    
  def round_to(num)
      #Debug      UI.messagebox("origional no =  " + num.to_s)
      num_R = ((num + (@rnd_Factor/2.0))/@rnd_Factor).to_i*@rnd_Factor
      num_R = num_R.to_l
      #Debug      UI.messagebox("rounded no =  " + num_R.to_s)
      return num_R
    end #round_to  
    
      
	def round_3places(n)  
		n=(n*1000).to_i
		n=n.to_f/1000      
	end #def round_3places
  
	def input_Offsets
		# open a file in read mode	       
		f = File.open(@file, "r")  
		ans = f.readlines
		# Close the file
		f.close
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
    space = round_to(@e_len1 - @end2CL)
      while n > 0
        prompts = ["CL Distance Domino " + i.to_s + " to Domino " + i2.to_s + " ","Remaining Space   READ ONLY  ","Number Spaces   READ ONLY  "]
        defaults= [@ans[15+i].to_s,space.to_s,n.to_s]
        results = inputbox prompts, defaults, "Input CL Distance between Biscuits"
        @cl_offset[i-1]=results[0].to_l; @ans[15+i] = results[0]
        space = round_to(space - @cl_offset[i-1])
        n = n-1
        i = i+1
        i2 = i2+1		
      end #while n > 0
		 # Write an array of Input Data to the file
      self.write_data         
	end #def input_Offsets 

	def cal_xCL_points(x1,e_len,dir_x)
		@xcl=Array.new @quanity.to_i
		case @spacing_Type         
			when "Auto"
				#Debug UI.messagebox("Case Spacing = " + @spacing_Type.to_s)
        if @quanity == 1
          @xcl[0]=x1+(@end2CL*dir_x)
        else
				equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
				@xcl[0]=x1+(@end2CL*dir_x)
				@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
				n =(@quanity-2)  #loop number
				sn = 1 #Starting number
				while n > 0
					@xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
					sn = sn + 1
					n = n -1
				end # while n > 0
        end # if @quanity
			#Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
			when "User"
				@xcl[0]=x1+(@end2CL*dir_x)
				n =(@quanity-1)  #loop number
				sn = 1 #Starting number
				i = 0
				while n > 0
					user_offset=@cl_offset[i]
					@xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
					sn = sn + 1
					i = i + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
		
		end #case @spacing_Type
		#Debug UI.messagebox("End Spacing = " + @spacing_Type.to_s)
	end #cal_xCL_points

	def points_DominoMortise_Comp1(x,y,z,r)
		@pt=Array.new 14
    #Debug UI.messagebox("@xf  = " + @xf.to_s)
		@pt[0]=Geom::Point3d.new((x+@xf)+r,y,z)
		@pt[1]=Geom::Point3d.new((x+@xf)+(0.866*r),y+(0.5*r),z)
		@pt[2]=Geom::Point3d.new((x+@xf)+(0.5*r),y+(0.866*r),z)
		@pt[3]=Geom::Point3d.new((x+@xf),y+r,z)
		@pt[4]=Geom::Point3d.new((x-@xf),y+r,z)
		@pt[5]=Geom::Point3d.new((x-@xf)-(0.5*r),y+(0.866*r),z)
		@pt[6]=Geom::Point3d.new((x-@xf)-(0.866*r),y+(0.5*r),z)
		@pt[7]=Geom::Point3d.new((x-@xf)-r,y,z)
		@pt[8]=Geom::Point3d.new((x-@xf)-(0.866*r),y-(0.5*r),z)
		@pt[9]=Geom::Point3d.new((x-@xf)-(0.5*r),y-(0.866*r),z)
		@pt[10]=Geom::Point3d.new((x-@xf),y-r,z)
		@pt[11]=Geom::Point3d.new((x+@xf),y-r,z)
		@pt[12]=Geom::Point3d.new((x+@xf)+(0.5*r),y-(0.866*r),z)
		@pt[13]=Geom::Point3d.new((x+@xf)+(0.866*r),y-(0.5*r),z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_DominoMortise_Comp1
  
	def points_DominoCL(x,y)
		@clpt=Array.new 4
		cline=0.03.inch
		@clpt[0]=Geom::Point3d.new(x+(cline),y,0)
		@clpt[1]=Geom::Point3d.new(x,y,0)
		@clpt[2]=Geom::Point3d.new(x,y+(cline),0)
		@clpt[3]=Geom::Point3d.new(x,y,0)
	end #def points_DominoCL

  def find_DominoComp(model,entities,definitions)
    length = definitions.length
    dname = ""
    i=0
    loop do
      i += 1
      break if i==length
      definition = definitions[length-i]
      dname = definition.name
      break if dname == @domino_name
    end  
      if dname == @domino_name
      @domino_exist = "yes"
      @domino_definition = definitions[length-i]
    end #if dname
  end #find_DominoComp
  
  def points_DominoCL(x,y) 
    @clpt=Array.new 4
    @clpt[0]=Geom::Point3d.new(x,y,0)
    @clpt[1]=Geom::Point3d.new(x+1,y,0)
    @clpt[2]=Geom::Point3d.new(x,y+0.5,0)
    @clpt[3]=Geom::Point3d.new(x,y,2)
    #Debug @clpt.each {|e| UI.messagebox("CL point Domino = "+ e.to_s)}
    end #def points_DominoCL
  
  def points_Domino(x,y,z,r,dxf)
		@ptd=Array.new 14
    #Debug UI.messagebox("@xf  = " + @xf.to_s)
		@ptd[0]=Geom::Point3d.new((x+dxf)+r,y,z)
		@ptd[1]=Geom::Point3d.new((x+dxf)+(0.866*r),y+(0.5*r),z)
		@ptd[2]=Geom::Point3d.new((x+dxf)+(0.5*r),y+(0.866*r),z)
		@ptd[3]=Geom::Point3d.new((x+dxf),y+r,z)
		@ptd[4]=Geom::Point3d.new((x-dxf),y+r,z)
		@ptd[5]=Geom::Point3d.new((x-dxf)-(0.5*r),y+(0.866*r),z)
		@ptd[6]=Geom::Point3d.new((x-dxf)-(0.866*r),y+(0.5*r),z)
		@ptd[7]=Geom::Point3d.new((x-dxf)-r,y,z)
		@ptd[8]=Geom::Point3d.new((x-dxf)-(0.866*r),y-(0.5*r),z)
		@ptd[9]=Geom::Point3d.new((x-dxf)-(0.5*r),y-(0.866*r),z)
		@ptd[10]=Geom::Point3d.new((x-dxf),y-r,z)
		@ptd[11]=Geom::Point3d.new((x+dxf),y-r,z)
		@ptd[12]=Geom::Point3d.new((x+dxf)+(0.5*r),y-(0.866*r),z)
		@ptd[13]=Geom::Point3d.new((x+dxf)+(0.866*r),y-(0.5*r),z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_Domino

  def draw_Domino(entities)
    @wm=@wt; @wf=@wm-@tt; dxf=@wf/2;
    x=0; y=0; z=0; r=@tt/2; pp=@lt/2
    points_Domino(x,y,z,r,dxf) #@pt
    #@ptd=@ptd.collect {|point| point.transform(backt)} #Transform to Comp 1
    #@ptd=@ptd.collect {|point| point.transform(@tform)} #Transform to Global
    #Add Dominos
    dface=entities.add_face(@ptd) 
    dface.pushpull(-1*pp)
    dface.pushpull(pp)
  end # draw_Domino
	
    
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select 1st Component for Domino Joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1 
		Sketchup::set_status_text("Select First Edge in component 1 Parallel to Domino Width:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities;@c1_layer = @comp1.layer
				@status=1 
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end # if@comp1			
		elsif @status==1	#select 2nd component
			ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
			if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp2); @ent2=@comp2.definition.entities;  @status=2
				@tform2 = @comp2.transformation.inverse
				onMouseMove(flags, x, y, view)
				@sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
			else
				UI.messagebox "Selection invalid, must be component"
			end #@entity			
		elsif @status==2 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@tform = @ip.transformation
					@edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position; @e_len1=@edge1.length 
					@status = 3
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid Use Right Click & Hide Interfering Comp"
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid "	
			end #if(@ip.valid?)
		elsif @status==3 #select short edge, thickness
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position; @e_len2=@edge2.length  
					if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
						orig=@e1sp
						p1=@e1sp
						p2=@e1ep
					end #if ((@e1sp==
					if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
						orig=@e1ep
						p2=@e1sp
						p1=@e1ep
					end  #if ((@e1ep==
					if orig==@e2sp
						p3=@e2ep
						else
						p3=@e2sp
					end #if orig

          #Data Input Section
          self.same_Joint
          if @use_ExistingData == "No"
            self.select_Domino
            self.data_Input_Form
            self.no_of_Offset_Input
            if @noOffsets >= 1 then self.input_Offsets end #@noOffsets
          else
            begin
              self.readExistingData
            rescue
              UI.messagebox("Problem reading data! ReEnter Data")
              self.select_Domino
              self.data_Input_Form
              self.no_of_Offset_Input
              if @noOffsets >= 1 then self.input_Offsets end #@noOffsets
            end # begin
          end # if @use_ExistingData

          
					#find face that has edges 1 and 2
					face=@edge1.common_face @edge2
					if (face==nil)                    #get valid face
						@oops=4
					else 
						#Debug UI.messagebox"got valid face"
						#get points in local coord system for face
						#x axis parallel to @edge1
						x,y,z=face.normal.axes
						#get vector parallel to @edge1              			         			          
						ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
						xx=ar.normalize
						yy=z.cross xx
						localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
						backt=Geom::Transformation.axes(orig,xx,yy,z)
						edges=face.outer_loop.edges
						eind=edges.index(@edge1)
						e_len=@edge1.length
						#Points on face in local coordinates
						p1=p1.transform(localt)
						p2=p2.transform(localt)
						p3=p3.transform(localt)           
						x1=p1.x
						x2=p2.x
						y1=p1.y
						y2=p3.y
						if x1>x2 then dir_x = -1 end
						if x1<x2 then dir_x = +1 end
						if y1<y2 then dir_y = +1 end
						if y1>y2 then dir_y = -1 end
            
						#Get domino data
						dominoDimensions
						@r=@tt/2 #radius of Domino Slot
						self.cal_xCL_points(x1,e_len,dir_x)  #Returns @xcl=Array.new @quanity
						#Debug  @xcl.each {|element| UI.messagebox("Cal @xcl Points = "+ element.to_s)}
            @ycl = y1+(@edge2CL*dir_y)
						#Debug UI.messagebox("yc1 = "+ @ycl.to_s)
            
            # Options for Complete Joint or Each Part of the Joint
            select_Tool_Options
            if @add_DomSlots_Comp1 == "Yes"
              Sketchup.active_model.start_operation "Domino Slot Comp1"
              i=0
              n=@quanity
              while n > 0
                #Calculate points for Domino Slot Face							
                points_DominoMortise_Comp1(@xcl[i],@ycl,0,@r) #@pt=Array.new 14
                #Transform Domino Slot Face Points to Component 1 Coordinates
                @ptc1=@pt.collect {|point| point.transform(backt)}
                #Add Domino Slot Face
                @entity=@comp1
                codef=@entity.definition
                @sel.clear
                @sel.add(@comp1)
                cface=codef.entities.add_face(@ptc1) #Fixed array so it uses all elements!
                cface.pushpull(-1*@mdC1)
                i=i+1
                n=n-1
                #Debug  UI.messagebox("Number Holes Left = " + n.to_s )
              end #while n > 0
              Sketchup.active_model.commit_operation
            end # add_DomSlots_Comp1 Feature added to First component at this point            

            if @add_DomSlots_Comp2 == "Yes"
              Sketchup.active_model.start_operation "Domino Slot Comp2"
              i=0
              n=@quanity
              while n > 0
                @entity=@comp2
                codef=@entity.definition
                @sel.clear
                @sel.add(@comp2)
                #Calculate points for Domino Slot Face							
                points_DominoMortise_Comp1(@xcl[i],@ycl,0,@r) #@pt=Array.new 14
                @ptc1=@pt.collect {|point| point.transform(backt)} # Points in Comp1 Coordinates
                @ptg=@ptc1.collect {|point| point.transform(@tform)} #Points in Global Coordinates
                #Debug @ptg.each {|e| UI.messagebox("point c1g = "+ e.to_s)}
                @ptC2=@ptg.collect {|point| point.transform(@tform2)} #Points in Comp2 Coordinates
                #Debug @clptC2.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}	            
                cface=codef.entities.add_face(@ptC2)
                cface.pushpull(-1*@mdC2)
                i=i+1
                n=n-1
                #Debug  UI.messagebox("Number Holes Left = " + n.to_s )
              end #while n > 0
              Sketchup.active_model.commit_operation
            end #if @add_DomSlots_Comp2 Feature added to second component at this point						
	
            if @addDomino == "Yes" 
              @sel.clear
              Sketchup.active_model.start_operation "Dominos"
              model = Sketchup.active_model
              entities = model.active_entities
              definitions = model.definitions
              layers = model.layers
              group = entities.add_group
              entities = group.entities
              @domino_exist = "no"
              find_DominoComp(model,entities,definitions)
              if @domino_exist == "no" #Domino component does not exist
                draw_Domino(entities) #Adds Domino @(0,0,0)
                component_instance=group.to_component
                component_instance.definition.name=(@domino_name)
                definition = component_instance.definition
              else
                definition = @domino_definition
              end
              i=0
              n=@quanity
              while n > 0
                #Calculate z for correct Domino position
                if @d_Comp1 != @d_length/2
                  domino_z = (@d_length/2) - @d_Comp1
                else
                  domino_z = 0
                end #if @c2depth 
                #Calculate CL of Hole
                domino_ip=Geom::Point3d.new([@xcl[i],@ycl,domino_z])								
                domino_ip=domino_ip.transform(backt)					
                domino_ip=domino_ip.transform(@tform)
                #Get points for domino axis at center of hole
                points_DominoCL(@xcl[i],@ycl)
                @clpt=@clpt.collect {|point| point.transform(backt)}
                @clpt=@clpt.collect {|point| point.transform(@tform)}
                #Debug @clpt.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}
                #Create vectors for domino axis
                x_vector = @clpt[0].vector_to @clpt[1]
                y_vector = @clpt[0].vector_to @clpt[2]
                z_vector = @clpt[0].vector_to @clpt[3]
                #Invoke Geom::Transformation.axes with the new origin, x-axis, y-axis, and z-axis
                f_tran = Geom::Transformation.axes(domino_ip,x_vector,y_vector,z_vector)
                #For i = 0 move existing component to first hole!
                if i == 0 && @domino_exist == "no"
                  #Debug UI.messagebox("f_tran = " + f_tran.to_s)
                  component_instance.transform! f_tran
                  component_instance.layer = @c1_layer
                else
                  if i == 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                  if i == 0 then instance2.layer = @c1_layer end
                end
                #For i > 0 Insert new instance of Domino at Domino axis origin
                if i > 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                if i > 0 then instance2.layer = @c1_layer end
                i=i+1
                n=n-1
              end #while
              Sketchup.active_model.commit_operation
						end # addDomino
       
            @comp2.hidden=false
						#Program is Complete and ends
						Sketchup.send_action "selectSelectionTool:"
					end #face==nil                        
				end	#if(@ip.edge &&
			else
				UI.messagebox "Selection invalid"	
			end #if(@ip.valid?)	
		end #elsif @status==
	end # def onLButtonDown
	
	def onRButtonDown(flags,x,y,view)
		reset
		self.dialog
		self.no_of_Offset_Input
		if @noOffsets >= 1
			input_Offsets
		end
	end

	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
end #end class K2_Tools_DominoJoint

 
end #module

 file_loaded("K2_Tools_DominoJoint.rb")
 
