


#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # 
  # Name :        K2_ToolsDowelJoint.rb  ###
  # Date :	      July 2014
  # Version :     2.1.0
  # Author :      Keith Krueger (with a big help from plugin forum)
  #
  
require 'sketchup.rb'
require 'K2WS_Tools/k2ws_module.rb' #-NameSpace Module

module K²WS 
  class K2_Tools_Dowel_Joint  
    
    def initialize
      @mod = Sketchup.active_model
      @sel = @mod.selection
      @opt_LEN_No = Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
      @opt_Format_No = Sketchup.active_model.options["UnitsOptions"]["LengthFormat"]
      @opt_Precision_No = Sketchup.active_model.options["UnitsOptions"]["LengthPrecision"]
      @ans=Array.new
      #Test for Acceptable Units/Format and Set rnd_Factor
      @rnd_Factor = 0
      if @opt_LEN_No == 0 && @opt_Format_No == 0 then @rnd_Factor = 10.0**-(1*@opt_Precision_No) end
      if @opt_LEN_No == 0 && @opt_Format_No == 3 then @rnd_Factor = 0.015625 * 2**(6 - @opt_Precision_No) end
      if @opt_LEN_No == 2 && @opt_Format_No == 0 then @rnd_Factor = 0.0393701 * 10**-(1*@opt_Precision_No) end
      if @rnd_Factor == 0
        UI.messagebox("Units must be Inches or Millimeters \n Fromat must be Decimal or Fractions \n \n CHANGE UNITS & RE-SELECT TOOL ")
        Sketchup.send_action "selectSelectionTool:"
      else
        #Debug   UI.messagebox("@rnd_Factor = " + @rnd_Factor.to_s)
      end #if @rnd_Factor
            
      @ip = Sketchup::InputPoint.new
      @dowel_Version = "2.2.6"
      @no_sides = 12
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file)) 
        f = File.new(@file, "w+") 
        f.close
        self.reset
      else
        self.reset
        self.read_data
        self.readExistingData
      end # FileTest
    end  #def
    
    def read_data
      # Read an array of Input Data from the file 
      # open input data file in read mode	       
      f = File.open(@file, "r")    
      @ans = f.readlines
      #strip end of line or file from all data
      @ans = @ans.collect { |data|
        data.strip 
      }
      # Close the file
      f.close		
    end #read_data
    
    def write_data
      # Write an array of Input Data to the file
      # open input data file in write mode	       
      f = File.open(@file, "w")
      f.puts @ans
      # Close the file
      f.close
    end
    
    def readExistingData
      self.read_data
      @fixture_Spacing = @ans[0]
      @hole_Depth_C1 = @ans[1].to_l
      @hole_Depth_C2 = @ans[2].to_l
      @style = @ans[3]
      @end2CL = @ans[4].to_l
      @quanity = @ans[5].to_i
      @dia = @ans[6].to_l
      @d_length = @ans[7].to_l
      @d_Comp1 = @ans[8].to_l
      @spacing_Type = @ans[9]
      @no_sides = @ans[10].to_i
      @edge2CL = @ans[11].to_l
      @thru_Hole = @ans[12]
      @edge1_Last = @ans[13].to_l
      @edge2_Last = @ans[14].to_l
      @use_ExistingData = @ans[15]
     
      self.no_of_Offset_Input
      n = @noOffsets
      @cl_offset=Array.new n
      i = 1
      i2 = 2
      while n > 0
        @cl_offset[i-1]=(@ans[15+i].to_s.strip.to_l)     
        n = n-1
        i = i+1
        i2 = i2+1		
      end #while n > 0
     
    end #readExistingData
    
    def reset
      @best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @dowel_name = nil
      @addDowels = "No";  @add_DowelHoles_Comp2 = "No"; @add_DowelHoles_Comp1 = "No"
    end
    
    def same_Joint
      begin
        @edge1_Last = @ans[13].to_l; @edge2_Last = @ans[14].to_l
      rescue
        @edge1_Last = 0; @edge2_Last = 0
        #Debug UI.messagebox("blank data error rescued")
      end
      if @e_len1 == @edge1_Last && @e_len2 == @edge2_Last
        result = UI.messagebox('Use Existing Dowel Joint Data ???', MB_YESNO)
        if result == 6 
          @use_ExistingData = "Yes"; @ans[15] = "No" # @use_ExistingData saved as No
        end # if result
      else
        @use_ExistingData = "No"; @ans[15] = "No"
      end # if @e_len1
      # Save current edge1 & edge2 Length
      @ans[13] = @e_len1.to_s;  @ans[14] = @e_len2.to_s
      write_data
    end # same_Joint
    
    def select_Dowel
    
    prompts = ["Joint Face READ ONLY  ","Style  ","Diameter","Dowel Length","No of Dowels"]
    defaults = [@e_len2.to_s + " x " + @e_len1.to_s,@ans[3],@ans[6].to_s,@ans[7].to_s,@ans[5].to_s]
    choices = ["","Smooth|Show Edges","","",""]
    results = inputbox prompts, defaults, choices, "Select Domino and Mortise Width"
    if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    @style = results[1].to_s; @ans[3] = results[1]
      if @style == "Show Edges" then @no_sides = 12 end
      if @style == "Smooth" then @no_sides = 24 end
      @ans[10] = @no_sides.to_s
    @dia = results[2].to_l; @ans[6] = results[2]
    @d_length = results[3].to_l; @ans[7] = results[3]
    @quanity = results[4].to_i; @ans[5] = results[4]
    write_data    
  
  end # select_Dowel
  
  
  def data_Input_Form
  
		# open input data file in read mode	       
		self.read_data
    cl_edge1 = round_to(@e_len1/2.0); cl_edge2 = round_to(@e_len2/2.0); end_toCL_first = round_to((@dia/2.0)+@dia)
    if @quanity == 1
      prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Dowel","End to CL Dowel","Dowel Depth Comp1","Through Dowel? "]
      defaults= [@e_len2.to_s + " x " + @e_len1.to_s,cl_edge2.to_s,cl_edge1.to_s,@ans[8].to_s,@ans[12]]
      choices = ["","","","","No|Comp 1|Comp 2|Both Comp"]
    else
      prompts = ["Joint Face READ ONLY  ","Edge 1 to CL Dowels","End to CL First Dowel","Dowel Depth Comp1","Through Dowel? ","Space Between Dowels"]
      defaults= [@e_len1.to_s + " x " + @e_len2.to_s,@ans[11].to_s,@ans[4].to_s,@ans[8].to_s,@ans[12].to_s,@ans[9]]
      choices = ["","","","","No|Comp 1|Comp 2|Both Comp","Auto|User|DowelMax|JessEm"]
    end # if @quanity
		results = inputbox prompts, defaults, choices, "Input Data for Dowel Joint (Dowel: " + @dia.to_s + " x " + @d_length.to_s + ")"
		if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    @edge2CL = results[1].to_l; @ans[11] = results[1]
    @end2CL = results[2].to_l; @ans[4] = results[2]
    @d_Comp1 = results[3].to_l; @ans[8] = results[3]
    @thru_Hole = results[4].to_s; @ans[12] = results[4]
      if @thru_Hole == "No"
        @hole_Depth_C1 = @d_Comp1; @ans[1] = @hole_Depth_C1
        @hole_Depth_C2 = @d_length - @d_Comp1; @ans[2] = @hole_Depth_C2
      else
        dowel_Thru_Hole_Depth
      end # if @thru_Hole 
    if @quanity >= 2
      @spacing_Type = results[5]; @ans[9] = results[5]
      case @spacing_Type
        when "DowelMax"
          @fixture_Spacing = 0.666      
        when "JessEm"
          if @dia == 0.375 then  @fixture_Spacing = 0.75 end
          if @dia == 0.250 then  @fixture_Spacing = 0.50 end
          if @dia == 0.500 then  @fixture_Spacing = 1.00 end
      end # case @spacing_Type
      @ans[0] = @fixture_Spacing.to_s
    else
      @spacing_Type = "Auto";  @ans[9] = "Auto"
      @fixture_Spacing = 0.666; @ans[0] = @fixture_Spacing.to_s
    end #if @quanity
		# Write an array of Input Data to the file
		self.write_data
  end #data_Input_Form
    
    def hole_Depth
      # open input data file in read mode	       
      @hole_Depth_C1 = @d_Comp1 + 0.0625.inch
      @hole_Depth_C2 = @d_length - @d_Comp1 + 0.0625.inch
      # Debug UI.messagebox(" Hole Depth Comp 2 = " + @hole_Depth_C2.to_s)
    end #hole_Depth
    
    def dowel_Thru_Hole_Depth

		if @thru_Hole == "Comp 1"
      prompts = ["Depth Thru Hole Comp 1 "]
      defaults= [@ans[1].to_s]
      choices = [""]
      results = inputbox prompts, defaults, choices, "Through Hole Depth COMPONENT 1"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @hole_Depth_C1 = results[0].to_l; @ans[1] = results[0]
      @hole_Depth_C2 = @d_length - @d_Comp1
    elsif @thru_Hole == "Comp 2"
      prompts = ["Depth Thru Hole Comp 2 "]
      defaults= [@ans[2].to_s]
      choices = [""]
      results = inputbox prompts, defaults, choices, "Through Hole Depth COMPONENT 2"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @hole_Depth_C1 = @d_Comp1
      @hole_Depth_C2 = results[0].to_l; @ans[2] = results[0]
    elsif @thru_Hole == "Both Comp"
      prompts = ["Depth Thru Hole Comp 1 ","Depth Thru Hole Comp 2 "]
      defaults= [@ans[1].to_s,@ans[2].to_s]
      choices = ["",""]
      results = inputbox prompts, defaults, choices, "Through Hole Depth Both COMPONENTS"
      if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
      @hole_Depth_C1 = results[0].to_l; @ans[1] = results[0]
      @hole_Depth_C2 = results[1].to_l; @ans[2] = results[1]
    end #@thru_Hole
		# open input data file in write mode	       
		self.write_data
    
	end #dowel_Thru_Hole_Depth 
  
  def select_Tool_Options
    @add_DowelHoles_Comp1 = "No"
    @add_DowelHoles_Comp2 = "No"
    @addDowels = "No"
    prompts = ["Select Tool Option"]
    defaults= ["Complete Joint"]
    choices = ["Complete Joint|Comp 1 Holes + Dowels|Add Dowels|Comp1 Holes|Comp 2 Holes|Comp 1 & 2 Holes|comp 2 Holes + Dowels_"]
    results = inputbox prompts, defaults, choices, "Select Dowel Joint Tool Options"
    if results == false then self.reset end; Sketchup.send_action "selectSelectionTool:"
    case results[0]
      when "Complete Joint"
        @add_DowelHoles_Comp1 = "Yes"
        @add_DowelHoles_Comp2 = "Yes"
        @addDowels = "Yes"
      when "Comp 1 Holes + Dowels"
        @add_DowelHoles_Comp1 = "Yes"
        @addDowels = "Yes"
      when "Add Dowels"
        @addDowels = "Yes"
      when "Comp1 Holes"
        @add_DowelHoles_Comp1 = "Yes"
      when "Comp 2 Holes"
        @add_DowelHoles_Comp2 = "Yes"
      when "Comp 1 & 2 Holes"
        @add_DowelHoles_Comp1 = "Yes"
        @add_DowelHoles_Comp2 = "Yes"
      when "comp 2 Holes + Dowels_"
        @add_DowelHoles_Comp2 = "Yes"
        @addDowels = "Yes"
    end # case results
  end # select_Tool_Options

    def no_of_Offset_Input
      case @spacing_Type    
        when "Auto"
          @noOffsets=0
        else
          @noOffsets=@quanity - 1
      end #case @spacing_Type
      self.write_data
    end #no_of_offsets
    
    def round_to(num)
      #Debug      UI.messagebox("origional no =  " + num.to_s)
      num_R = ((num + (@rnd_Factor/2.0))/@rnd_Factor).to_i*@rnd_Factor
      num_R = num_R.to_l
      #Debug      UI.messagebox("rounded no =  " + num_R.to_s)
      return num_R
    end #round_to
    
    def round_3places(n)  
      n=(n*1000).to_i
      n=n.to_f/1000      
    end #def round_3places
    
    def input_Offsets
      # open a file in read mode	       
      self.read_data
      n = @noOffsets
      @cl_offset=Array.new n
      i = 1
      i2 = 2
      space = round_to(@e_len1 - @end2CL)
      if @spacing_Type == "Auto" || @spacing_Type == "User"
        while n > 0
          prompts = ["CL Distance Dowel " + i.to_s + " to Dowel " + i2.to_s + " ","Remaining Space   READ ONLY  ","Number Spaces   READ ONLY  "]
          defaults= [@ans[15+i].to_s,space.to_s,n.to_s]
          results = inputbox prompts, defaults, "Input CL Distance between Biscuits"
          @cl_offset[i-1]=results[0].to_l; @ans[15+i] = results[0]
          space = round_to(space - @cl_offset[i-1])
          n = n-1
          i = i+1
          i2 = i2+1		
        end #while n > 0
      else
        while n > 0
          no_FixtureSpaces = (@ans[15+i].to_l/@fixture_Spacing.to_f).to_i
          prompts = ["No of Fixture Spaces:  Dowel " + i.to_s + " to Dowel " + i2.to_s + " "]
          defaults= [no_FixtureSpaces.to_s]
          results = inputbox prompts, defaults, "Input CL Distance between Biscuits"
          @cl_offset[i-1]=(results[0].to_i * @fixture_Spacing.to_f).to_l; @ans[15+i] = (results[0].to_i * @fixture_Spacing).to_s
          space = round_to(space - @cl_offset[i-1])
          n = n-1
          i = i+1
          i2 = i2+1		
        end #while n > 0
      end
      # Write an array of Input Data to the file
      self.write_data      
    end #def input_Offsets 

    def cal_xCL_points(x1,e_len,dir_x)
      @xcl=Array.new @quanity.to_i
      case @spacing_Type          
        when "Auto"
          #Debug UI.messagebox("Case Spacing = " + @spacing_Type.to_s)
          if @quanity == 1
            @xcl[0]=x1+(@end2CL*dir_x)
          else
          equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
          @xcl[0]=x1+(@end2CL*dir_x)
          @xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
          n =(@quanity-2)  #loop number
          sn = 1 #Starting number
          while n > 0
            @xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
            sn = sn + 1
            n = n -1
          end # while n > 0
          end # if @quanity
            #Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
        else
          #Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
          @xcl[0]=x1+(@end2CL*dir_x)
          n =(@quanity-1)  #loop number
          sn = 1 #Starting number
          i = 0
          while n > 0
            user_offset=@cl_offset[i]
            #user_offset=user_offset.to_l
            #Debug UI.messagebox("user_offset = "+ user_offset.to_s)
            @xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
            sn = sn + 1
            i = i + 1
            n = n -1
          end # while n > 0
          #Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
      end #case @spacing_Type
      #Debug UI.messagebox("End Spacing = " + @spacing.to_s)
    end #cal_xCL_points
    
    def points_DowelCircle_Comp1(x,y,z,r)
      #r=@dia/2
      #@no_sides = 24
      @pt=Array.new @no_sides
      #Points using Math formula
      for k in 0...@no_sides
        xsf = Math.sin(((Math::PI*2)/@no_sides)*k)
        ycf = Math.cos(((Math::PI*2)/@no_sides)*k)
        #Debug UI.messagebox("sin = " + xsf.to_s + "  cos = " + ycf.to_s)
        @pt[k]=Geom::Point3d.new(x+(xsf*r),y+(ycf*r),z)
      end #for k
      #Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
    end #points_DowelCircle
    
    def points_DowelCL(x,y) 
      @clpt=Array.new 4
      @clpt[0]=Geom::Point3d.new(x,y,0)
      @clpt[1]=Geom::Point3d.new(x+1,y,0)
      @clpt[2]=Geom::Point3d.new(x,y+0.5,0)
      @clpt[3]=Geom::Point3d.new(x,y,2)
      #Debug @clpt.each {|e| UI.messagebox("CL point Dowel = "+ e.to_s)}
    end #def points_DowelCL
    
    def dowel_name
      #dowel name based on dia and length
      @dowel_name = ("Part Dowel_" + @style + "_" + @dia.to_s + "_" + @d_length.to_s)
    end #dowel_name
    
    def drawNew_Dowel(entities,definitions,layers)
        #Add Arrays and Variables for creating dowel with Beveled Ends
        dp1=Array.new @no_sides; dp2=Array.new @no_sides;	dp3=Array.new @no_sides; dp4=Array.new @no_sides			 
        bevel=@dia/6
        rse=((@dia/2)-bevel)  #radius small end of dowel
        r=@dia/2
        z1=(@d_length/2);	z2=((@d_length/2)-bevel)
        z4=z1*(-1); z3=z2*(-1)
        #Draw Dowel @ 0,0,z
        points_DowelCircle_Comp1(0,0,z1,rse); dp1=@pt #Bevel End		
        points_DowelCircle_Comp1(0,0,z2,r); dp2=@pt #Full Dia		
        points_DowelCircle_Comp1(0,0,z3,r); dp3=@pt #Full Dia		
        points_DowelCircle_Comp1(0,0,z4,rse); dp4=@pt #Bevel End		
        points_DowelCL(0,0)
        dface=entities.add_face(dp1)
        dface.reverse!
        dface=entities.add_face(dp4)
        #dface.reverse!
        end_k = @no_sides - 1
        for k in -1...end_k
          k2=k+1
          dface=entities.add_face(dp1[k],dp2[k],dp2[k2],dp1[k2])
          edges = dface.edges
          if @style == "Smooth"
            #edges.each {|e|e.smooth="true"}
            edges.each {|e|e.soft="true"}
          end # if @style
          dface=entities.add_face(dp2[k],dp3[k],dp3[k2],dp2[k2])
          edges = dface.edges
          if @style == "Smooth"
            #edges.each {|e|e.smooth="true"}
            edges.each {|e|e.soft="true"}
          end #if @style
          dface=entities.add_face(dp3[k],dp4[k],dp4[k2],dp3[k2])
          edges = dface.edges
          if @style == "Smooth"
            #edges.each {|e|e.smooth="true"}
            edges.each {|e|e.soft="true"}
          end #if @style
        end #for k
      end #drawNew_Dowel
    
    def find_DowelComp(model,entities,definitions)
      length = definitions.length
      dname = ""
      i=0
      loop do
        i += 1
        break if i==length
        definition = definitions[length-i]
        dname = definition.name
        break if dname == @dowel_name
      end  
        if dname == @dowel_name
        @dowel_exist = "yes"
        @dowel_definition = definitions[length-i]
      end #if dname
    end #find_DowelComp
         
    def onMouseMove(flags, x, y, view)
      Sketchup::set_status_text("Select 1st Component for Dowel Joint: OR  [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
      Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1
      Sketchup::set_status_text("Select First Edge in component 1 Parallel to Dowel Pattern:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
      Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
      @ip.pick view,x,y
      view.tooltip = @ip.tooltip     
      view.refresh
    end
    
    def onLButtonDown(flags, x, y, view)
      if @status==0 #select 1st component
        ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
        if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
          @sel.add(@comp1); @ent1=@comp1.definition.entities; @c1_layer = @comp1.layer
          @status=1; if @toolType == "SingleComp_Tool" then @status=2 end
          onMouseMove(flags, x, y, view)
        else
          UI.messagebox "Selection invalid, must be component"
        end #@comp1			
      elsif @status==1	#select 2nd component
        ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
        if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
          @sel.add(@comp2); @ent2=@comp2.definition.entities; @status=2
          if @comp2.definition == @comp1.definition then @toolType = "SingleComp_Tool" end
          @tform2 = @comp2.transformation.inverse
          onMouseMove(flags, x, y, view)
          @sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
        else
          UI.messagebox "Selection invalid, must be component"
        end #@entity			
      elsif @status==2 #select long edge, width
        @ip.pick view,x,y
        if(@ip.valid?)
          if(@ip.edge && @ent1.include?(@ip.edge))
            @tform = @ip.transformation
            @edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position; @e_len1=@edge1.length 
            @status = 3
            onMouseMove(flags, x, y, view)
          else
            UI.messagebox "Selection invalid "
          end #if(@ip.edge
        else
          UI.messagebox "Selection invalid "	
        end #if(@ip.valid?)
      elsif @status==3 #select short edge, thickness
        @ip.pick view,x,y
        if(@ip.valid?)
          if(@ip.edge && @ent1.include?(@ip.edge))
            @edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position; @e_len2=@edge2.length 
            if @comp2 != nil then @comp2.hidden=false end
            if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
              orig=@e1sp
              p1=@e1sp
              p2=@e1ep
            end #if ((@e1sp==
            if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
              orig=@e1ep
              p2=@e1sp
              p1=@e1ep
            end  #if ((@e1ep==
            if orig==@e2sp
              p3=@e2ep
            else
              p3=@e2sp
            end #if orig
            
            #Data Input Section
            self.same_Joint
            if @use_ExistingData == "No"
              self.select_Dowel
              self.data_Input_Form
              #self.hole_Depth
              self.no_of_Offset_Input
              if @noOffsets >= 1 then input_Offsets end #@noOffsets
            else
              #self.hole_Depth
            end # if @use_ExistingData
            
            #find face that has edges 1 and 2
            face=@edge1.common_face @edge2
            @face_zaxis_vector = face.normal           
            if (face==nil)  # Quits without valid face
              @oops=4
            else 
              #Debug UI.messagebox"got valid face"
                          
              #get points in local coord system for face
              #xaxis parallel to @edge1
              x,y,z=face.normal.axes
              #get vector parallel to @edge1              			         			          
              ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
              xx=ar.normalize
              yy=z.cross xx
              @face_xaxis_vector = xx
              @face_yaxis_vector = yy
              @face_origin = orig
              localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
              backt=Geom::Transformation.axes(orig,xx,yy,z)
              edges=face.outer_loop.edges
              eind=edges.index(@edge1)
              e_len=@edge1.length
              #Points on face in local coordinates
              p1=p1.transform(localt)
              p2=p2.transform(localt)
              p3=p3.transform(localt)           
              x1=p1.x
              x2=p2.x
              y1=p1.y
              y2=p3.y
              if x1>x2 then dir_x = -1 end
              if x1<x2 then dir_x = +1 end
              if y1<y2 then dir_y = +1 end
              if y1>y2 then dir_y = -1 end
              #UI.messagebox("Dir Y, Dir X  =" + dir_y.to_s + " " + dir_x.to_s)
              self.cal_xCL_points(x1,e_len,dir_x)
              #Debug @xcl.each {|e| UI.messagebox("Symetrical CL point = "+ e.to_s)}
              
              #Joint Features Added as Specified by User
              self.select_Tool_Options
              if @add_DowelHoles_Comp1 == "Yes"
                i=0
                n=@quanity
                Sketchup.active_model.start_operation "Dowel Holes"                
                while n > 0               			          
                  #@xcl.each {|element| UI.messagebox("Cal @xcl Points = "+ element.to_s)}
                  ycl = y1+(@edge2CL*dir_y)
                  r=@dia/2
                  points_DowelCircle_Comp1(@xcl[i],ycl,0,r) # Returns @pt=Array.new @no_sides
                  #Debug UI.messagebox"Returned from Calculating Points for Dowel Circle "
                  #Calculate points for CLPoint of Dowel Circle
                  #Transform Dowel Circle Points to Component 1 Coordinates
                  @pt=@pt.collect {|point| point.transform(backt)}
                  #Add Dowel Circle Face
                  @comp1.hidden=false
                  @entity=@comp1
                  codef=@entity.definition
                  @sel.clear
                  @sel.add(@comp1)
                  cface=codef.entities.add_face(@pt) #Fixed array so it uses all elements!
                  orig_ents = codef.entities.to_a
                  cface.pushpull(-1*@hole_Depth_C1)
                  fin_ents = codef.entities.to_a
                  added_ents = fin_ents - orig_ents
                  #added_ents.each {|e| e.smooth="true" if e.is_a?(Sketchup::Edge) } #Bad for thru holes 
                  added_ents.each {|e| e.soft="true" if e.is_a?(Sketchup::Edge) }
                  #Feature added to First component at this point
                  i=i+1
                  n=n-1
                end #while n > 0
                # Dowel Holes added to Component 1 at this Point
                Sketchup.active_model.commit_operation
              end # add_DowelHoles_Comp1
                           
              if @add_DowelHoles_Comp2 == "Yes"
                i=0
                n=@quanity
                Sketchup.active_model.start_operation "Dowel Holes"                
                while n > 0               			          
                  @entity=@comp2
                  codef=@entity.definition
                  @sel.clear
                  @sel.add(@comp2)
                  ycl = y1+(@edge2CL*dir_y)
                  r=@dia/2
                  points_DowelCircle_Comp1(@xcl[i],ycl,0,r) # Returns @pt=Array.new @no_sides
                  @pt=@pt.collect {|point| point.transform(backt)} # transforms to Comp1
                  @ptg=@pt.collect {|point| point.transform(@tform)} # transforms to Global
                  @ptC2=@ptg.collect {|point| point.transform(@tform2)} # transforms to comp2
                  #Debug @clptC2.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}	            
                  cface=codef.entities.add_face(@ptC2)
                  orig_ents = codef.entities.to_a
                  edges = cface.edges				
                  cface.pushpull(-1*@hole_Depth_C2)
                  fin_ents = codef.entities.to_a
                  added_ents = fin_ents - orig_ents
                  #added_ents.each {|e| e.smooth="true" if e.is_a?(Sketchup::Edge) } #Bad for thru holes 
                  added_ents.each {|e| e.soft="true" if e.is_a?(Sketchup::Edge) }
                  #Feature added to second component at this point               
                  i=i+1
                  n=n-1
                end #while n > 0
                Sketchup.active_model.commit_operation
                #"Dowel Holes Added to Second Component"
              end #	add_DowelHoles_Comp2
              
              
              if @addDowels == "Yes"
                @sel.clear
                Sketchup.active_model.start_operation "Dowels"
                model = Sketchup.active_model
                entities = model.active_entities
                definitions = model.definitions
                layers = model.layers
                group = entities.add_group
                entities = group.entities
                # Get existing Dowel or Draw a Dowel Component
                dowel_name
                @dowel_exist = "no"
                find_DowelComp(model,entities,definitions)
                if @dowel_exist == "no" #dowel component does not exist
                  drawNew_Dowel(entities,definitions,layers) #Adds Dowel @(0,0,0)
                  component_instance=group.to_component
                  component_instance.definition.name=(@dowel_name)
                  component_instance.layer = @c1_layer
                  definition = component_instance.definition
                else
                  definition = @dowel_definition
                end
                i=0
                n=@quanity
                while n > 0
                  ycl = y1+(@edge2CL*dir_y)
                  r=@dia/2
                  #Calculate z for correct dowel position
                  if @d_Comp1 != @d_length/2
                    dowel_z = (@d_length/2) - @d_Comp1
                  else
                    dowel_z = 0
                  end #if @c2depth 
                  #Calculate CL of Hole
                  dowel_ip=Geom::Point3d.new([@xcl[i],ycl,dowel_z])								
                  dowel_ip=dowel_ip.transform(backt)								
                  dowel_ip=dowel_ip.transform(@tform)
                  #Get points for dowel axis at center of hole
                  points_DowelCL(@xcl[i],ycl)
                  @clpt=@clpt.collect {|point| point.transform(backt)}
                  @clpt=@clpt.collect {|point| point.transform(@tform)}
                  #Debug @clpt.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}
                  #Create vectors for dowel axis
                  x_vector = @clpt[0].vector_to @clpt[1]
                  y_vector = @clpt[0].vector_to @clpt[2]
                  z_vector = @clpt[0].vector_to @clpt[3]
                  #Invoke Geom::Transformation.axes with the new origin, x-axis, y-axis, and z-axis
                  f_tran = Geom::Transformation.axes(dowel_ip,x_vector,y_vector,z_vector)
                  #For i = 0 move existing component to first hole!
                  if i == 0 && @dowel_exist == "no"
                    component_instance.transform! f_tran
                  else
                    if i == 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                    if i == 0 then instance2.layer = @c1_layer end
                  end
                  #For i > 0 Insert new instance of dowel at dowel axis origin
                  if i > 0 then instance2=model.active_entities.add_instance(definition,f_tran) end
                  if i > 0 then instance2.layer = @c1_layer end
                  i=i+1
                  n=n-1
                end #while
              # Dowels added to Joint at this Point
              Sketchup.active_model.commit_operation
              end # addDowels
              #Program is Complete and ends
              if @comp2 != nil
                if @comp2.definition == @comp1.definition then UI.messagebox("You selected INSTANCES of the SAME COMPONENT it may be necessary to FLIP a COMPONENT TO COMPLETE JOINT!")	end
              end
              Sketchup.send_action "selectSelectionTool:"
            end #face==nil
          else
              UI.messagebox "Selection invalid "
          end #if(@ip.edge
        else
          UI.messagebox "Selection invalid"	
        end #if(@ip.valid?)	
      end #elsif @status==
    end # def onLButtonDown
    
    def onCancel(flag, view)
      view.refresh if @drawn
      Sketchup.send_action "selectSelectionTool:"
    end

    def draw(view)
      @drawn = false
      if( @ip.valid? && @ip.display? )
      @ip.draw(view)
      @drawn = true
      end
    end
    
    def onRButtonDown(flags,x,y,view)
      reset
      self.data_Input_Form
      self.no_of_Offset_Input
      if @noOffsets >= 1
        input_Offsets
      end
    end

  end #end class K2_Tools_Dowel_Joint

end #name space module

#File KK_ToolsLoader.rb is used to Load this file

 file_loaded("K2_Tools_DowelJoint.rb")
 
